"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const ora_1 = tslib_1.__importDefault(require("ora"));
const foam_core_1 = require("foam-core");
const write_file_to_disk_1 = require("../utils/write-file-to-disk");
const utils_1 = require("../utils");
class Janitor extends command_1.Command {
    async run() {
        const spinner = ora_1.default('Reading Files').start();
        const { args, flags } = this.parse(Janitor);
        const { workspacePath = './' } = args;
        if (utils_1.isValidDirectory(workspacePath)) {
            const config = foam_core_1.createConfigFromFolders([foam_core_1.URI.file(workspacePath)]);
            const services = {
                dataStore: new foam_core_1.FileDataStore(config),
            };
            const graph = (await foam_core_1.bootstrap(config, services)).notes;
            const notes = graph.getNotes().filter(Boolean); // removes undefined notes
            spinner.succeed();
            spinner.text = `${notes.length} files found`;
            spinner.succeed();
            // exit early if no files found.
            if (notes.length === 0) {
                this.exit();
            }
            spinner.text = 'Generating link definitions';
            const fileWritePromises = notes.map(note => {
                // Get edits
                const heading = foam_core_1.generateHeading(note);
                const definitions = foam_core_1.generateLinkReferences(note, graph, !flags['without-extensions']);
                // apply Edits
                let file = note.source.text;
                file = heading ? foam_core_1.applyTextEdit(file, heading) : file;
                file = definitions ? foam_core_1.applyTextEdit(file, definitions) : file;
                if (heading || definitions) {
                    return write_file_to_disk_1.writeFileToDisk(note.uri, file);
                }
                return Promise.resolve(null);
            });
            await Promise.all(fileWritePromises);
            spinner.succeed();
            spinner.succeed('Done!');
        }
        else {
            spinner.fail('Directory does not exist!');
        }
    }
}
exports.default = Janitor;
Janitor.description = 'Updates link references and heading across all the markdown files in the given workspaces';
Janitor.examples = [
    `$ foam-cli janitor path-to-foam-workspace
`,
];
Janitor.flags = {
    'without-extensions': command_1.flags.boolean({
        char: 'w',
        description: 'generate link reference definitions without extensions (for legacy support)',
    }),
    help: command_1.flags.help({ char: 'h' }),
};
Janitor.args = [{ name: 'workspacePath' }];
